/* 
 * Copyright (C) 2008 Ondrej Stanek, ostan89<at>gmail.com 
 * http://www.ostan.cz/pocketbot
 *   
 * All rights reserved.
*/
#include <avr/io.h>
#include "NEC_IR_Rx.h"
#include "RemoteControl_buttons.h"
#include "main.h"
#include "effectors.h"
#include "IRsensors.h"
#include "lineFollowing.h"

#define SET_MOTORS 0x12
#define SET_PID	0x13
#define SET_SPEED 0x14

#define MAXSPEED 255
#define	SPEED2	170
#define	SPEED3	110

inline void ManualControlButtons (uint8_t key) {
	switch(key) {
		case POWER_ON:	
			pocketBotState=SETTINGS;
			LED_SetMode(BLUE,LED_SETTINGS_MODE);
			break;
		
		// remote control
		case STOP:
			OCR1AL = MAXSPEED;
			OCR1BL = SPEED2;
			break;
		
		case SKIP_L:
			OCR1AL = SPEED2;
			OCR1BL = 0;
			break;

		case MEM:
			OCR1AL = SPEED3;
			OCR1BL = 0;
			break;
		
		case REPEAT:
			OCR1AL = MAXSPEED;
			OCR1BL = MAXSPEED;
			break;

		case X_BASS:
			OCR1AL = SPEED2;
			OCR1BL = SPEED2;
			break;

		case RAND:
			OCR1AL = SPEED3;
			OCR1BL = SPEED3;
			break;

		case PLAY_PAUSE:
			OCR1AL = SPEED2;
			OCR1BL = MAXSPEED;
			break;

		case SKIP_R:
			OCR1AL = 0;
			OCR1BL = SPEED2;
			break;

		case BAND:
			OCR1AL = 0;
			OCR1BL = SPEED3;
			break;
	}
}

inline void SettingsButtons (uint8_t key) {
	switch(key) {	
		case DISC_SKIP:	
			pocketBotState=MANUAL;
			LED_SetMode(BLUE,LED_MANUAL_CONTROL_MODE);
			break;
		
		case PLAY_PAUSE:
			pocketBotState=FOLLOWING;
			break;
		
		case STOP:
			pocketBotState=SETTINGS;
			break;

		//regulator adjustments
		case SKIP_R:
			regulator.d++;
			break;

		case BAND:
			regulator.d--;
			break;

		case X_BASS:
			regulator.p++;
			break;

		case RAND:
			regulator.p--;
			break;

		case VOL_UP:
			regulator.speed+=6;
			break;
		case VOL_DOWN:
			regulator.speed-=6;
			break;

		//calibration adjustments
		case SOUND:
			startConversion();
			callbackAfterConversion=UpdateCalibrationOffset;
			break;
		case TIMER:
			startConversion();
			callbackAfterConversion=UpdateCalibrationGain;
			break;
		case OPEN: //loads all constants form EEPROM
			LoadCalibrationDataFromEEPROM();
			LoadRegulatorDataFromEEPROM();
			break;
		case MEM: //stores all calibration to EEPROM
			WriteCalibrationDataToEEPROM();
			WriteRegulatorDataToEEPROM();
	}
}

void ProccesNECdata(remoteCommand_t command, uint8_t repeatCode) {

	if( (command[0] == 67) && (command[1] == 71) ) { //command from our remote control
		LED_Blink(RED, LED_IR_COMMAND);
		//if(command[2] == ~command[3]) //check for packet consistency //not used, caused errors(?)
		if(pocketBotState==MANUAL) {
			ManualControlButtons(command[2]);
		} else {
			if (!repeatCode) //every button press is evaluated only once, even when the signal is repetitive
				SettingsButtons(command[2]);
		}
		remoteControl_off_timeout = F_CPU / 256 / 256 * 0.120; //Every 110 ms a repeat code is send. Lets give it 10ms reserve..

	} else { //propably command form pc
		switch (command[0]) {
		 	case SET_MOTORS:
				LED_Blink(RED, LED_PC_COMMAND);
				OCR1A = command[2];
				OCR1B = command[3];
				remoteControl_off_timeout = F_CPU / 256 / 256 * 0.120; //Every 110 ms a repeat code is send. Lets give it 10ms reserve..
				break;
			case SET_PID:
				regulator.p = command[1];
				regulator.i = command[2];
				regulator.d = command[3];
				break;
			case SET_SPEED:
				regulator.speed = command[1];	
		}
	}
	

}
