/* 
 * Copyright (C) 2008 Ondrej Stanek, ostan89<at>gmail.com 
 * http://www.ostan.cz/pocketbot
 *   
 * All rights reserved.
*/

#include <avr/io.h>
#include "effectors.h"

void motors_init(void) {

	TCCR1A = 1<<COM1A1 | 1<<COM1B1 | 1<<WGM10; //Phase Correct PWM
	TCCR1B = 1<<CS12 | 0<<CS10; //prescaler 64 (is this value ok??)
	DDRB |= 1<<PB1 | 1<<PB2; //set OCR1A and OCR1B as outputs

}


// this module implements asynchronous controlling of LED,
// which gains programmer the possibility of showing current
// status clearly without using any blocking functions

uint8_t sw_timer0;
#define SW_TIMER0_MAX	10

struct {
	uint8_t blinking_mode;
	uint8_t blink;
}	leds[2];

// default led blinking mode
void LED_SetMode(leds_t led, uint8_t mode) {
	leds[led].blinking_mode = mode;
}

// blink a message,
void LED_Blink(leds_t led, uint8_t b) {
	leds[led].blink = b;
	sw_timer0=SW_TIMER0_MAX-1; //should appear immidiately
}

void LED_Controller_init(void) {
	DDRB |= 1<<PB6 | 1<<PB7; //set LED pins as outputs
	
	LED_SetMode(BLUE, LED_SETTINGS_MODE);
	LED_Blink(RED, LED_POWER_ON);


}	


void LED_Controller_Poll(void) {
	sw_timer0++;
	if (sw_timer0 == SW_TIMER0_MAX) {
		sw_timer0 = 0;
		static uint8_t progress;
		
		progress = (progress+1)&7;		

		if (leds[RED].blink) {
			if (bit_is_set(leds[RED].blink,0)) {
				RED_LED_ON();
			} else {
				RED_LED_OFF();
			}
			leds[RED].blink=leds[RED].blink>>1; 
		} else {
			if (bit_is_set(leds[RED].blinking_mode,progress)) {
				RED_LED_ON();
			} else {
				RED_LED_OFF();
			}
		}


		if (leds[BLUE].blink) {
			if (bit_is_set(leds[BLUE].blink,0)) {
				BLUE_LED_ON();
			} else {
				BLUE_LED_OFF();
			}
			leds[BLUE].blink=leds[BLUE].blink>>1; 
		} else {
			if (bit_is_set(leds[BLUE].blinking_mode,progress)) {
				BLUE_LED_ON();
			} else {
				BLUE_LED_OFF();
			}
		}

	}
}
