/* 
 * Copyright (C) 2008 Ondrej Stanek, ostan89<at>gmail.com 
 * http://www.ostan.cz/pocketbot
 *   
 * All rights reserved.
*/
#include <avr/io.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include "lineFollowing.h"
#include "IRsensors.h"
#include "effectors.h"

#define TRESHOLD 150 //this value sets sensitivity, it decides what to do if contrast is bad, f.e. when the line is at the edge of sensor module

line_t line;

regulator_t regulator;
regulator_t regulator_EEMEM; //CAUTION!! Propably this is wrong, it should be <regulator_t EEMEM regulator_EEMEM> to allocate data to EEPROM

int16_t previousLinePosition=0;
float sumPositions=0;

void LoadRegulatorDataFromEEPROM(void) {
	eeprom_read_block(&regulator, &regulator_EEMEM, sizeof(regulator));
}

void WriteRegulatorDataToEEPROM(void) {
 	eeprom_write_block(&regulator, &regulator_EEMEM, sizeof(regulator));
}


void getLinePosition(void) {
	uint8_t n;
	uint8_t lowest=0;

	//find the lowest value
	for (n=0; n<6; n++) 
		if (sensors[n]<sensors[lowest]) lowest=n;

	if (sensors[lowest]<TRESHOLD) {
		line.position=0;
		line.found=1;
		for (n=0; n<lowest; n++) 
			line.position+=sensors[n];
		for (n=lowest+1; n<6; n++)
			line.position-=sensors[n];
	} else {
		line.found=0;
	}
}

//this is the maximum value of line.position
#define MAX_LINE_POSITION (255*5)

uint8_t spiral=0;
uint8_t obstacle=0;
uint8_t rotate=0;

void adjustMotorSpeeds(void) {

	if (sensors[6]==255) {
		obstacle=20;
		if (line.position<0) rotate=1;
			else rotate=0;
	}

	if (obstacle) {
		obstacle--;
		if (rotate) {
			OCR1AL=200;
			OCR1BL=0;
		}	else {
			OCR1AL=0;
			OCR1BL=200;
		}
	} else {

		if (line.found) {
			spiral=0;

			
      int32_t correction = (int32_t)line.position * (int32_t)regulator.p
                          + (int32_t)(line.position - previousLinePosition) * ((int32_t)regulator.d) * 8L;//*((int32_t)regulator.p-127L);
      
			correction = correction * regulator.speed;
      int32_t denominator = (int32_t)(MAX_LINE_POSITION * 64L); //64 is "the significant point of 100%"
      //int32_t denominator = 81600L; //64 is "the significant point of 100%"
      int32_t correction_normalized = correction / denominator ; 
			
      uint8_t pomalejsi;
      if(abs(correction_normalized) < regulator.speed) {
      	pomalejsi = regulator.speed - abs(correction_normalized);
      } else {
        pomalejsi = 0;
      }
      if(correction_normalized>0) {
      	OCR1AL = pomalejsi; //pravy
        OCR1BL = regulator.speed; //levy
      } else {
      	OCR1AL = regulator.speed;
        OCR1BL = pomalejsi;
      }
      
      previousLinePosition=line.position;
      //sumPositions+=line.position;
		


		} else {
			RED_LED_ON(); //show that line was lost
		
			//spiral trajectory seeking for a line
			spiral++;
			sumPositions=0;

			if (spiral>=regulator.speed) {
				spiral=regulator.speed-1;
			}
			if (line.position<0) {
				OCR1AL=regulator.speed;
				OCR1BL=spiral;
			}	else {
				OCR1AL=spiral;
				OCR1BL=regulator.speed;
			}
		}
	}
}
