/* 
 * Copyright (C) 2008 Ondrej Stanek, ostan89<at>gmail.com 
 * http://www.ostan.cz/pocketbot
 *   
 * All rights reserved.
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "IRsensors.h"
#include "uart.h"
#include "NEC_IR_Rx.h"
#include "effectors.h"
#include "main.h"
#include "pocketBot_protocol.h"
#include "lineFollowing.h"

pocketBotState_t pocketBotState=SETTINGS;

inline void mainTimer_init(void) {

	TCCR0 |= 1<<CS02; //prescaler 256 (122Hz overflow)
	TIMSK |= 1<<TOIE0; //enable overflow interrupt

}


uint8_t remoteControl_off_timeout;
uint8_t timer0;

ISR(TIMER0_OVF_vect) {
	if (pocketBotState==FOLLOWING) {
		timer0++;
		if (timer0==2) {
			timer0=0;
			startConversion(); //this will start the process of measuring ended with PID algorithm
		}
	} else {

		if (remoteControl_off_timeout) {
			remoteControl_off_timeout--;
		} else {
			OCR1AL = 0;
			OCR1BL = 0;
		}
	}
//	sei(); //following code isn't time critical, therefore we enable global interrupts
	LED_Controller_Poll();
}

int main(void) {
	mainTimer_init();
	sensors_init();
  UART_init();
	remoteControl_init();
	motors_init();
	LED_Controller_init();
	
	LoadCalibrationDataFromEEPROM();
	LoadRegulatorDataFromEEPROM();
		
	sei();
	
  while(1) {
		pocketBotBufferPoll();

/* this code is out of date, USART routines aren't implemented any more, use TxBuffer instead!
#ifdef RC_DEBUG
	
		if (rcError) {
			if (rcError==RECEIVING_COMPLETED) {
				Send_IRcommand();
			}
			if (rcError==REPEAT_CODE) {
				Send_IRcommand();
			}
	  	PORTB ^= 1<<PB7; //test blinking when executing interrupt
			char buffer[4];
			itoa(errStep,buffer,10);
			USART_PutS(buffer);
			USART_Put('e');
			itoa(rcError,buffer,10);
			USART_PutS(buffer);
			USART_Put('t');
			itoa(tcnt_val,buffer,10);
			USART_PutS(buffer);
			USART_Put(' ');
			rcError=OK;
			sei();
		}
		
#endif
*/
	}

	return 0;
}


ISR(__vector_default) { //catching unwanted interrupts
	LED_Blink(RED,LED_UNWANTED_INTERRUPT);


}
