/* 
 * Copyright (C) 2008 Ondrej Stanek, ostan89<at>gmail.com 
 * http://www.ostan.cz/pocketbot
 *   
 * All rights reserved.
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h> 

#include "uart.h"
#include "pocketBot_protocol.h"
#include "effectors.h"

#define BAUDRATE 38400 //communication speed in Bauds. Higher speeds aren't reliable
#define UBRRVAL ((F_CPU/(BAUDRATE*16UL))-1) //count the UBRR value

#define TX_BUFF_SIZE_MASK	0b01111111 //128 bytes buffer

uint8_t txBuf[TX_BUFF_SIZE_MASK+1];
uint8_t txBtm=0;
uint8_t txTop=0; //relative pointers to top and bottom of the buffer


void UART_init(void)
{

	//Set baud rate
	UBRRL=UBRRVAL;		//low byte
	UBRRH=(UBRRVAL>>8);	//high byte
	//Set data frame format: asynchronous mode,no parity, 1 stop bit, 8 bit size
	UCSRC=(1<<URSEL)|(0<<UMSEL)|(0<<UPM1)|(0<<UPM0)|(0<<USBS)|(0<<UCSZ2)|(1<<UCSZ1)|(1<<UCSZ0);	
	//Enable Transmitter and Receiver and Interrupt on receive complete
	UCSRB=(1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<UDRIE);
	
	PORTD |= 1<<PD0; //pullup for RXD
	
}


void TxNext(void) {
	UDR = txBuf[txBtm];
	txBtm=(txBtm+1)&TX_BUFF_SIZE_MASK;
}


/* CAUTION! THIS BUFFER SOLUTION MAY DROP SOME BYTE ONCE UPON A TIME, 
I GUESS THE PROBLEM MAY BE CAUSED BY INTERRUPT DRIVEN TRANSCIEVING  */
void TxBufAppend(uint8_t data)
{
 txBuf[txTop] = data;
 txTop = (txTop+1) & TX_BUFF_SIZE_MASK;
 if(txTop==txBtm) {
   LED_SetMode(RED, LED_BUFFER_OVERFLOW); //buffer overflow, this should not happen
	 txBtm = (txBtm+1) & TX_BUFF_SIZE_MASK; //drop the oldest byte
 }
 if (bit_is_clear(UCSRB,UDRIE)) { //if interrupt is disabled
		UCSRB |= 1<<UDRIE; //enable interrupt, which will start processing data buffer
	}

}


ISR(USART_UDRE_vect) {
	if(txBtm!=txTop) {
		TxNext();
	} else {
		UCSRB &= ~(1<<UDRIE);//disable interrupt
	}
}

ISR(USART_RXC_vect) 
{
	//UDR register MUST be once read, no matter if the value is used or not. Ingoring this will cause this interrupt routine to be called infinitely
	PBP_bufferByte(UDR);
}
